@@ -409,7 +409,6 @@ class Group(HLObject, MutableMappingHDF5):
             values are stored as scalar datasets. Raise ValueError if we
             can't understand the resulting array dtype.
         """
-        do_link = False
         with phil:
             name, lcpl = self._e(name, lcpl=True)
 
@@ -421,7 +420,9 @@ class Group(HLObject, MutableMappingHDF5):
                               lcpl=lcpl, lapl=self._lapl)
 
             elif isinstance(obj, ExternalLink):
-                do_link = True
+                fn = filename_encode(obj.filename)
+                self.id.links.create_external(name, fn, self._e(obj.path),
+                                              lcpl=lcpl, lapl=self._lapl)
 
             elif isinstance(obj, numpy.dtype):
                 htype = h5t.py_create(obj, logical=True)
@@ -431,12 +432,6 @@ class Group(HLObject, MutableMappingHDF5):
                 ds = self.create_dataset(None, data=obj)
                 h5o.link(ds.id, self.id, name, lcpl=lcpl)
 
-        if do_link:
-            fn = filename_encode(obj.filename)
-            with phil:
-                self.id.links.create_external(name, fn, self._e(obj.path),
-                                              lcpl=lcpl, lapl=self._lapl)
-
     @with_phil
     def __delitem__(self, name):
         """ Delete (unlink) an item from this group. """
@@ -453,6 +448,12 @@ class Group(HLObject, MutableMappingHDF5):
         for x in self.id.__iter__():
             yield self._d(x)
 
+    @with_phil
+    def __reversed__(self):
+        """ Iterate over member names in reverse order. """
+        for x in self.id.__reversed__():
+            yield self._d(x)
+
     @with_phil
     def __contains__(self, name):
         """ Test if a member name exists """
