@@ -18,15 +18,16 @@ import os
 import stat
 import pickle
 import tempfile
-from sys import platform
+import subprocess
+import sys
 
 from .common import ut, TestCase, UNICODE_FILENAMES, closed_tempfile
 from h5py import File
 import h5py
 from .. import h5
-
 import pathlib
 
+
 class TestFileOpen(TestCase):
 
     """
@@ -42,7 +43,6 @@ class TestFileOpen(TestCase):
             with File(fname):
                 pass
 
-
         # Existing readonly file; open read-only
         with File(fname, 'w'):
             pass
@@ -143,8 +143,9 @@ class TestFileOpen(TestCase):
         with self.assertRaises(ValueError):
             File(self.mktemp(), 'mongoose')
 
+
 @ut.skipIf(h5py.version.hdf5_version_tuple < (1, 10, 1),
-               'Requires HDF5 1.10.1 or later')
+           'Requires HDF5 1.10.1 or later')
 class TestSpaceStrategy(TestCase):
 
     """
@@ -182,6 +183,60 @@ class TestSpaceStrategy(TestCase):
         dset[...] = 1
         fid.close()
 
+
+@ut.skipIf(h5py.version.hdf5_version_tuple < (1, 10, 1),
+           'Requires HDF5 1.10.1 or later')
+@pytest.mark.mpi_skip
+class TestPageBuffering(TestCase):
+    """
+        Feature: Use page buffering
+    """
+
+    def test_only_with_page_strategy(self):
+        """Allow page buffering only with fs_strategy="page".
+        """
+        fname = self.mktemp()
+        with File(fname, mode='w', fs_strategy='page', page_buf_size=16*1024):
+            pass
+        with self.assertRaises(OSError):
+            File(fname, mode='w', page_buf_size=16*1024)
+        with self.assertRaises(OSError):
+            File(fname, mode='w', fs_strategy='fsm', page_buf_size=16*1024)
+        with self.assertRaises(OSError):
+            File(fname, mode='w', fs_strategy='aggregate', page_buf_size=16*1024)
+
+    def test_check_page_buf_size(self):
+        """Verify set page buffer size, and minimum meta and raw eviction criteria."""
+        fname = self.mktemp()
+        pbs = 16 * 1024
+        mm = 19
+        mr = 67
+        with File(fname, mode='w', fs_strategy='page',
+                  page_buf_size=pbs, min_meta_keep=mm, min_raw_keep=mr) as f:
+            fapl = f.id.get_access_plist()
+            self.assertEqual(fapl.get_page_buffer_size(), (pbs, mm, mr))
+
+    def test_too_small_pbs(self):
+        """Page buffer size must be greater than file space page size."""
+        fname = self.mktemp()
+        fsp = 16 * 1024
+        with File(fname, mode='w', fs_strategy='page', fs_page_size=fsp):
+            pass
+        with self.assertRaises(OSError):
+            File(fname, mode="r", page_buf_size=fsp-1)
+
+    def test_actual_pbs(self):
+        """Verify actual page buffer size."""
+        fname = self.mktemp()
+        fsp = 16 * 1024
+        pbs = 2 * fsp
+        with File(fname, mode='w', fs_strategy='page', fs_page_size=fsp):
+            pass
+        with File(fname, mode='r', page_buf_size=pbs-1) as f:
+            fapl = f.id.get_access_plist()
+            self.assertEqual(fapl.get_page_buffer_size()[0], fsp)
+
+
 class TestModes(TestCase):
 
     """
@@ -240,6 +295,12 @@ class TestDrivers(TestCase):
         self.assertEqual(fid.driver, 'stdio')
         fid.close()
 
+        # Testing creation with append flag
+        fid = File(self.mktemp(), 'a', driver='stdio')
+        self.assertTrue(fid)
+        self.assertEqual(fid.driver, 'stdio')
+        fid.close()
+
     @ut.skipUnless(os.name == 'posix', "Sec2 driver is supported on posix")
     def test_sec2(self):
         """ Sec2 driver is supported on posix """
@@ -248,6 +309,12 @@ class TestDrivers(TestCase):
         self.assertEqual(fid.driver, 'sec2')
         fid.close()
 
+        # Testing creation with append flag
+        fid = File(self.mktemp(), 'a', driver='sec2')
+        self.assertTrue(fid)
+        self.assertEqual(fid.driver, 'sec2')
+        fid.close()
+
     def test_core(self):
         """ Core driver is supported (no backing store) """
         fname = self.mktemp()
@@ -257,6 +324,12 @@ class TestDrivers(TestCase):
         fid.close()
         self.assertFalse(os.path.exists(fname))
 
+        # Testing creation with append flag
+        fid = File(self.mktemp(), 'a', driver='core')
+        self.assertTrue(fid)
+        self.assertEqual(fid.driver, 'core')
+        fid.close()
+
     def test_backing(self):
         """ Core driver saves to file when backing store used """
         fname = self.mktemp()
@@ -312,7 +385,6 @@ class TestDrivers(TestCase):
         with self.assertRaises(ValueError):
             File(tf, 'w', driver='core')
 
-
     # TODO: family driver tests
 
 
@@ -398,7 +470,7 @@ class TestNewLibver(TestCase):
         f.close()
 
     @ut.skipIf(h5py.version.hdf5_version_tuple < (1, 11, 4),
-           'Requires HDF5 1.11.4 or later')
+               'Requires HDF5 1.11.4 or later')
     def test_single_v112(self):
         """ Opening with "v112" libver arg """
         f = File(self.mktemp(), 'w', libver='v112')
@@ -447,7 +519,6 @@ class TestUserblock(TestCase):
         with self.assertRaises(ValueError):
             File(self.mktemp(), 'w', userblock_size='non')
 
-
     def test_write_only(self):
         """ User block only allowed for write """
         name = self.mktemp()
@@ -630,6 +701,7 @@ class TestClose(TestCase):
         f.close()
         f.close()
 
+
 class TestFlush(TestCase):
 
     """
@@ -718,7 +790,6 @@ class TestCloseInvalidatesOpenObjectIDs(TestCase):
         assert not g2.id.valid
 
 
-
 class TestPathlibSupport(TestCase):
 
     """
@@ -754,6 +825,7 @@ class TestPickle(TestCase):
 # hence no subclassing TestCase
 @pytest.mark.mpi
 class TestMPI(object):
+    @pytest.mark.skip(reason="fixture 'mpi_file_name' not present")
     def test_mpio(self, mpi_file_name):
         """ MPIO driver and options """
         from mpi4py import MPI
@@ -762,8 +834,18 @@ class TestMPI(object):
             assert f
             assert f.driver == 'mpio'
 
+    @pytest.mark.skip(reason="fixture 'mpi_file_name' not present")
+    def test_mpio_append(self, mpi_file_name):
+        """ Testing creation of file with append """
+        from mpi4py import MPI
+
+        with File(mpi_file_name, 'a', driver='mpio', comm=MPI.COMM_WORLD) as f:
+            assert f
+            assert f.driver == 'mpio'
+
+    @pytest.mark.skip(reason="fixture 'mpi_file_name' not present")
     @pytest.mark.skipif(h5py.version.hdf5_version_tuple < (1, 8, 9),
-        reason="mpio atomic file operations were added in HDF5 1.8.9+")
+                        reason="mpio atomic file operations were added in HDF5 1.8.9+")
     def test_mpi_atomic(self, mpi_file_name):
         """ Enable atomic mode for MPIO driver """
         from mpi4py import MPI
@@ -773,6 +855,7 @@ class TestMPI(object):
             f.atomic = True
             assert f.atomic
 
+    @pytest.mark.skip(reason="fixture 'mpi_file_name' not present")
     def test_close_multiple_mpio_driver(self, mpi_file_name):
         """ MPIO driver and options """
         from mpi4py import MPI
@@ -784,7 +867,7 @@ class TestMPI(object):
 
 
 @ut.skipIf(h5py.version.hdf5_version_tuple < (1, 10, 1),
-               'Requires HDF5 1.10.1 or later')
+           'Requires HDF5 1.10.1 or later')
 class TestSWMRMode(TestCase):
 
     """
@@ -814,6 +897,71 @@ class TestSWMRMode(TestCase):
         fid.close()
 
 
+@pytest.mark.skipif(
+    h5py.version.hdf5_version_tuple < (1, 12, 1) and (
+    h5py.version.hdf5_version_tuple[:2] != (1, 10) or h5py.version.hdf5_version_tuple[2] < 7),
+    reason="Requires HDF5 >= 1.12.1 or 1.10.x >= 1.10.7")
+@pytest.mark.skipif("HDF5_USE_FILE_LOCKING" in os.environ,
+                    reason="HDF5_USE_FILE_LOCKING env. var. is set")
+class TestFileLocking:
+    """Test h5py.File file locking option"""
+
+    def test_reopen(self, tmp_path):
+        """Test file locking when opening twice the same file"""
+        fname = tmp_path / "test.h5"
+
+        with h5py.File(fname, mode="w", locking=True) as f:
+            f.flush()
+
+            # Opening same file in same process without locking is expected to fail
+            with pytest.raises(OSError):
+                with h5py.File(fname, mode="r", locking=False) as h5f_read:
+                    pass
+
+            with h5py.File(fname, mode="r", locking=True) as h5f_read:
+                pass
+
+            with h5py.File(fname, mode="r", locking='best-effort') as h5f_read:
+                pass
+
+    def test_unsupported_locking(self, tmp_path):
+        """Test with erroneous file locking value"""
+        fname = tmp_path / "test.h5"
+        with pytest.raises(ValueError):
+            with h5py.File(fname, mode="r", locking='unsupported-value') as h5f_read:
+                pass
+
+    def test_multiprocess(self, tmp_path):
+        """Test file locking option from different concurrent processes"""
+        fname = tmp_path / "test.h5"
+
+        def open_in_subprocess(filename, mode, locking):
+            """Open HDF5 file in a subprocess and return True on success"""
+            h5py_import_dir = str(pathlib.Path(h5py.__file__).parent.parent)
+
+            process = subprocess.run(
+                [
+                    sys.executable,
+                    "-c",
+                    f"""
+import sys
+sys.path.insert(0, {h5py_import_dir!r})
+import h5py
+f = h5py.File({str(filename)!r}, mode={mode!r}, locking={locking})
+                    """,
+                ],
+                capture_output=True)
+            return process.returncode == 0 and not process.stderr
+
+        # Create test file
+        with h5py.File(fname, mode="w", locking=True) as f:
+            f["data"] = 1
+
+        with h5py.File(fname, mode="r", locking=False) as f:
+            # Opening in write mode with locking is expected to work
+            assert open_in_subprocess(fname, mode="w", locking=True)
+
+
 # unittest doesn't work with pytest fixtures (and possibly other features),
 # hence no subclassing TestCase
 class TestROS3:
